@@ -88,15 +88,15 @@ estat(path, stbuf)
 	return r;
 }
 
-long
+off_t
 elseek(fd, pos, action)
 	int	fd, action;
-	long	pos;
+	off_t	pos;
 {
-	long	r;
+	off_t	r;
 
 	if ((r = lseek(fd, pos, action)) == -1) {
-		fprintf(stderr, "%s: lseek(%d, %d, %ld): %s\n", progname,
+		fprintf(stderr, "%s: lseek(%d, %ld, %d): %s\n", progname,
 			fd, pos, action, strerror(errno));
 	}
 	return r;
