@@ -54,8 +54,8 @@ hp_getaddr()
 char **
 hp_nextaddr()
 {
-	if (*alist == 0)
-		return 0;
+	if (*alist == NULL)
+		return NULL;
 	return ++alist;
 }
 
@@ -80,4 +80,4 @@ hp_addr_randomize(hp)
 	alist = oalist;
 }
 
-#endif	/* HAVE_RESOLVER */
+#endif	/* HAVE_NETDB_H */
