@@ -1,6 +1,8 @@
 /*
  *	Copyright 1990 by Rayan S. Zachariassen, all rights reserved.
  *	This will be free software, but only when it is finished.
+ *
+ *	Rewrite for GNU autoconf by  Matti Aarnio <mea@nic.funet.fi> 1996
  */
 
 #include "hostenv.h"
@@ -31,6 +33,12 @@ rfc822tz(timep, ts, prettyname)
 	else
 	  offset = -timezone;
 	offset /= 60;				/* Offset in minutes */
+#else
+#ifdef HAVE_TIMEZONE /* Has ``timezone'', but no ``altzone'' ?? */
+	offset = -timezone;
+	if ((*ts)->tm_isdst)
+	  offset += 3600; /* One hour! */
+	offset /= 60;				/* Offset in minutes */
 #else
 	{		/* This is fallback stuff, beware! */
 	  time_t tm;
@@ -41,6 +49,7 @@ rfc822tz(timep, ts, prettyname)
 	  offset = (tm - *timep) / 60; /* Offset in minutes */
 	}
 #endif
+#endif
 #endif
 	sign   = offset >= 0;
 	if (offset < 0)
