@@ -121,7 +121,7 @@ _hdr_compound(cp, n, cstart, cend, type, tp, tlist, tlistp)
 	char msgbuf[50];
 
 	if (*cp != cstart)
-		abort();
+		abort(); /* Sanity check!  Call fault! */
 	++cp, --n;
 nextline:
 	for (; n > 0 && *cp != cend; ++cp, --n) {
@@ -161,7 +161,7 @@ nextline:
 	} else if (*cp == cend) {	/* we found matching terminator */
 		--n;			/* move past terminator */
 	} else {	/* there was an error */
-	  abort() ;
+	  abort() ; /* ??? some sort of sanity check ? */
 	}
 	tp->t_type = type;
 	return n;
