@@ -9,6 +9,9 @@
  * ZENV file  SELFADDRESSES= -entry  as a string of style:
  *	"1.2.3.4,6.7.8.9,::1.2.3.4,my.domain.name,"
  *              ^-------^---------^----------  commas to separate them!
+ *
+ * In theory we COULD use  SIOCGIFCONF, but it is somewhat open-ended
+ * beast with its own troubles..
  */
 
 #include "hostenv.h"
@@ -16,9 +19,13 @@
 #ifdef HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
 #endif
+#ifdef HAVE_STDLIB_H
+#include <stdlib.h>
+#endif
 #include <netdb.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
+#include <string.h>
 
 extern char *getzenv __((const char *));
 
