@@ -344,11 +344,11 @@ mail_close(fp)
 	 * on NFS mounted postoffices if you want to guarantee not losing
 	 * data without being told about it.
 	 */
-	if (fflush(fp) == EOF
-/* #ifdef NFSFSYNC */ /* This is propably ALWAYS a good idea */
+	if (fflush(fp) != 0
+#ifdef HAVE_FSYNC
 	    || fsync(FILENO(fp)) < 0
-/* #endif */	/* NFSFSYNC */
-	    || fclose(fp) == EOF) {
+#endif
+	    || fclose(fp) != 0) {
 		mail_free(message);
 		errno = EIO;
 		return -1;
@@ -421,7 +421,7 @@ mail_close(fp)
 	 * it the hard way.
 	 */
 
-	if (link(message, nmessage) < 0) {
+	if (link(message, nmessage) != 0) {
 		eno = errno;
 		fprintf(stderr, "link(\"%s\", \"%s\"): errno %d\n",
 				message, nmessage, errno);
@@ -477,9 +477,9 @@ mail_close_alternate(fp,where,suffix)
 	 * data without being told about it.
 	 */
 	if (fflush(fp) == EOF
-/* #ifdef NFSFSYNC */ /* This is propably ALWAYS a good idea */
+#ifdef HAVE_FSYNC
 	    || fsync(FILENO(fp)) < 0
-/* #endif */	/* NFSFSYNC */
+#endif
 	    || fclose(fp) == EOF) {
 		mail_free(message);
 		errno = EIO;
@@ -505,7 +505,7 @@ mail_close_alternate(fp,where,suffix)
 	 * it the hard way.
 	 */
 
-	if (link(message, nmessage) < 0) {
+	if (link(message, nmessage) != 0) {
 		eno = errno;
 		fprintf(stderr, "link(\"%s\", \"%s\"): errno %d\n",
 				message, nmessage, errno);
