@@ -18,6 +18,14 @@ mail_alloc(nbytes)
 	return (char*)malloc(nbytes);
 }
 
+char *
+mail_realloc(oldptr,nbytes)
+	unsigned int nbytes;
+	char *oldptr;
+{
+	return (char*)realloc(oldptr,nbytes);
+}
+
 void
 mail_free(s)
 	char *s;
