@@ -273,7 +273,7 @@ findfreefd()
                 }
         fprintf(stderr, "%s: out of free filedescriptors (%d)!\n",
                         progname, fd);
-        abort();
+        abort(); /* Out of free file-descriptors! */
         /* NOTREACHED */
 	return 0;
 }
@@ -508,7 +508,7 @@ ioop(cmd, command, name, ioflags, defaultfd, arg1)
 	if (cmd == sIOopenPipe && !(ioflags & O_CREAT)) {
 		if ((defaultfd = pipefd) < 0) {
 			fprintf(stderr, "%s: no pipe!\n", progname);
-			abort();
+			abort(); /* No pipe on IOopenPipe when it should exist! */
 		}
 	} else {
 		if (cmd == sIOintoBuffer) {
@@ -1491,7 +1491,7 @@ interpret(code, eocode, entry, caller, retcodep, cdp)
 			if (variableIndex >= VARMAXDEPTH) {
 			  fprintf(stderr,"%s: interpret.c: varStack[] recursed once too many. Max depth: %d\n",
 				  progname, VARMAXDEPTH);
-			  abort();
+			  abort(); /* varStack[] recursed too deep! */
 			}
 
 			if (command->buffer == NULL)
@@ -1569,7 +1569,7 @@ interpret(code, eocode, entry, caller, retcodep, cdp)
 			if (commandIndex >= COMMANDMAXDEPTH) {
 			  fprintf(stderr,"%s: interpret.c: commandStack[] recursed once too many. Max depth: %d\n",
 				  progname, COMMANDMAXDEPTH);
-			  abort();
+			  abort(); /* commandStack[] recursed too deep! */
 			}
 			command->buffer = NULL;
 			command->bufferp = &command->buffer;
@@ -1854,7 +1854,7 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 				grindef("TMPO = ", l); */
 			} else {
 				fprintf(stderr, "parameter without call\n");
-				abort();
+				abort(); /* parameter without call! */
 			}
 			break;
 		case sAssign:
@@ -1999,7 +1999,7 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 			if (nloop >= LOOPMAXDEPTH) {
 			  fprintf(stderr,"%s: interpret.c: sLoopEnter once too many times, max depth: %d\n",
 				  progname, LOOPMAXDEPTH);
-			  abort();
+			  abort(); /* sLoopEnter too deep! */
 			}
 			loop[nloop].cont = 0;
 			loop[nloop].brk = 0;
@@ -2111,7 +2111,7 @@ XXX: HERE! Must copy the output to PREVIOUS memory level, then discard
 			if (++nsift >= (sizeof sift / sizeof sift[0])) {
 				fprintf(stderr,"%s: sSiftPush more than %d times allowed by the interpret.c code\n",
 					progname, nsift-1);
-				abort();
+				abort(); /* Too deep SIFTs! */
 			}
 			sift[nsift].kind  = 0;
 			sift[nsift].str = NULL;
@@ -2262,7 +2262,7 @@ std_printf("set %x at %d\n", re, cdp->rearray_idx);
 			if (++nsift >= (sizeof sift / sizeof sift[0])) {
 				fprintf(stderr,"%s: sTSiftPush more than %d times allowed by the interpret.c code\n",
 					progname, nsift-1);
-				abort();
+				abort(); /* Too deep TSIFT! */
 			}
 			sift[nsift].kind  = 1;
 			sift[nsift].tlist = NULL;
