@@ -21,9 +21,12 @@
 #include "shconfig.h"
 
 extern int vsiodoprnt();
+#define NO_FLOAT_CONVERSION
+#ifndef NO_FLOAT_CONVERSION
 extern int cvt();
 extern double modf();
 extern char *exponent(), *round();
+#endif
 #if !defined(__STDC__) && !defined(_AIX)
  extern int printf();
 #endif
@@ -58,7 +61,10 @@ extern int printf(); /* Sun 4.1.x with old GCC */
 #endif
 int (*std_printf) __((char *fmt, ...)) = (int(*)__((char *, ...)))printf;
 #else
-int (*std_printf) __((/*char *fmt, ...*/)) = (int (*) __((/* char *fmt, ... */)))printf;
+#if (defined(__GNUC__) && defined(sun) && !defined(__svr4__))
+extern int printf __((char *fmt, ...));
+#endif
+int (*std_printf) __((char *fmt, ...)) = (int (*) __((char *fmt, ...)))printf;
 #endif
 
 int std_fread(b,s,n,fp) char *b; int s, n; FILE *fp;{ return fread(b,s,n,fp); }
@@ -513,6 +519,15 @@ rflag:		switch (*++fmt) {
 			}
 			base = 10;
 			goto number;
+#ifdef NO_FLOAT_CONVERSION
+		case 'e':
+		case 'E':
+		case 'f':
+		case 'g':
+		case 'G':
+			_double = va_arg(argp, double);
+			break;
+#else
 		case 'e':
 		case 'E':
 		case 'f':
@@ -552,6 +567,7 @@ rflag:		switch (*++fmt) {
 				sign = '-';
 			t = *buf ? buf : buf + 1;
 			goto pforw;
+#endif
 		case 'n':
 			if (flags & LONGINT)
 				*va_arg(argp, long *) = cnt;
@@ -717,6 +733,7 @@ pforw:
 	/* NOTREACHED */
 }
 
+#ifndef NO_FLOAT_CONVERSION
 int
 cvt(number, prec, flags, signp, fmtch, startp, endp)
 	double number;
@@ -977,3 +994,4 @@ exponent(p, exp, fmtch)
 	}
 	return(p);
 }
+#endif
