@@ -284,6 +284,9 @@ modp_btree(sip)
 				sip->file);
 		return 0;
 	}
+	if (stbuf.st_nlink == 0)
+		return 1;	/* Unlinked underneath of us! */
+	
 	symid = symbol((u_char *)sip->file);
 	spl = sp_lookup(symid, spt_modcheck);
 	if (spl != NULL) {
