@@ -1483,6 +1483,22 @@ sequencer(e, file)
 	  }
 	}
 
+	dprintf("Nuke Return-Path headers if any\n");
+	oh = NULL;
+	for (h = e->e_headers; h != NULL; oh = h, h = h->h_next) {
+	  if (h->h_descriptor->hdr_name == NULL)
+	    continue;
+	  if ((h->h_descriptor->class == normal
+	       && CISTREQ(h->h_descriptor->hdr_name,"return-path"))
+	      || (h->h_descriptor->class == Resent
+		  && CISTREQ(h->h_descriptor->hdr_name,"resent-return-path"))){
+	    if (oh == NULL)
+	      e->e_headers = h->h_next;
+	    else
+	      oh->h_next = h->h_next;
+	  }
+	}
+
 	/*
 	 * Log the message after we find the envelope From address, otherwise
 	 * log entries might have empty sender fields.
@@ -2007,6 +2023,10 @@ sequencer(e, file)
 
 	if (vfp != NULL) {
 		fprintf(vfp, "router done processing %s\n", file);
+		fflush(vfp);
+#ifdef HAVE_FSYNC
+		fsync(FILENO(vfp));
+#endif
 		fclose(vfp);
 	}
 	
@@ -2022,7 +2042,11 @@ sequencer(e, file)
 	qpath = emalloc(5+strlen(QUEUEDIR)+strlen(file));
 #endif
 	sprintf(qpath, "../%s/%s", QUEUEDIR, file);
-	if ((fclose(ofp) == EOF) || (erename(file, qpath) < 0)) {
+	fflush(ofp);
+#ifdef HAVE_FSYNC
+	fsync(FILENO(ofp));
+#endif
+	if ((fclose(ofp) != 0) || (erename(file, qpath) != 0)) {
 		unlink(qpath);
 		unlink(ofpname);
 #ifndef	USE_ALLOCA
