@@ -12,8 +12,6 @@
 #include "scheduler.h"
 #include "prototypes.h"
 
-extern char *strchr();
-
 time_t qipcretry = 0;
 
 int	/* return non-zero when there are some childs
@@ -79,7 +77,7 @@ char *turnarg;
 	if (cp) *cp++ = 0;
 	/* If cp is non-null, some additional args were present.. */
 
-	spl = sp_lookup(symbol(turnarg), spt_mesh[L_HOST]);
+	spl = sp_lookup(symbol((void*)turnarg), spt_mesh[L_HOST]);
 	if (spl == NULL || (wp = (struct web *)spl->data) == NULL) {
 	  /* Not found, nothing to do.. */
 	  return;
