@@ -75,11 +75,14 @@ char	*rendezvous;
 char	*pidfile = PID_SCHEDULER;
 char	*mailshare;
 char	*logfn;
-static int mustexit;
-static int gotalarm;
-static int dumpq;
-static int canexit;
-static int rereadcf;
+int	slow_shutdown = 0;
+extern int readsockcnt; /* from transport.c: mux() */
+ 
+static int mustexit = 0;
+static int gotalarm = 0;
+static int dumpq    = 0;
+static int canexit  = 0;
+static int rereadcf = 0;
 static int dlyverbose = 0;
 time_t	sched_starttime;
 int	do_syslog = 0;
@@ -91,6 +94,8 @@ int	global_wrkcnt = 0;
 int	mailq_Q_mode = 0;
 int	syncstart = 0;		/* while set, the thread subsystem shall start
 				   no childs! */
+int	freeze = 0;		/* For debugging, complete disable of child
+				   running.. */
 int	hashlevels = 0;		/* How many levels of hashes are supported
 				   at the transport directory ?  (To speed
 				   up the directory processing, file opens,
@@ -127,6 +132,7 @@ extern void  cfp_mksubdirs __((char *, char*));
 extern char *cfpdirname __((int));
 
 static RETSIGTYPE sig_exit   __((int sig));
+static RETSIGTYPE sig_quit   __((int sig));
 static RETSIGTYPE sig_alarm  __((int sig));
 static RETSIGTYPE sig_iot    __((int sig));
 static RETSIGTYPE sig_readcf __((int sig));
@@ -250,8 +256,8 @@ main(argc, argv)
 	int i, daemonflg, c, errflg, version, fd;
 	long offout, offerr;
 	struct config_entry *tp;
-	int freeze = 0;
 
+	freeze = 0;
 	/* setlinebuf(stderr);  -- no need for this ? */
 
 	ArgvSave = argv;
@@ -497,6 +503,7 @@ main(argc, argv)
 
 	canexit = 1;
 	SIGNAL_HANDLE(SIGTERM, sig_exit);	/* split */
+	SIGNAL_HANDLE(SIGQUIT, sig_quit);	/* Slow shutdown */
 #ifdef	MALLOC_TRACE
 	mal_leaktrace(1);
 #endif	/* MALLOC_TRACE */
@@ -533,7 +540,7 @@ main(argc, argv)
 	    if (dirqueuescan(".", dirq, 
 			     (now >= next_idlecleanup)) < 150) {
 	      /* If we have more things to scan, don't quit yet! */
-	      next_dirscan = now + sweepinterval;	/* 10 seconds interval */
+	      next_dirscan = now + sweepinterval;  /* 10 seconds interval */
 	      if (dirq->wrksum > 100)
 		next_dirscan = now + 60; /* Lots of work, pick new jobs
 					    less frequently */
@@ -572,7 +579,7 @@ main(argc, argv)
 	  canexit = 1;
 
 	  /* mux on the stdout of children */
-	  do {
+	  for (;;) {
 	    if (dumpq) {
 	      qprint(-1);
 	      dumpq = 0;
@@ -582,8 +589,19 @@ main(argc, argv)
 	      rereadcf = 0;
 	    }
 	    mytime(&now);
-	  } while (mux(timeout) == 0 && !gotalarm &&
-		   dirq->wrksum == 0 && !mustexit && timeout > now);
+	    if (slow_shutdown) {
+	      timeout = now+2;
+	      shutdown_kids();  /* If there are any to shut down.. */
+	    }
+	    if (!(mux(timeout) == 0 && !gotalarm &&
+		  dirq->wrksum == 0 && !mustexit && timeout > now))
+	      break;
+	    if (readsockcnt == 0 && slow_shutdown) {
+	      /* No more childs to read, hop hop die away! */
+	      mustexit = 1;
+	      break;
+	    }
+	  }
 	} while (!mustexit);
 
 	/* Doing nicely we would kill the childs here, but we are not
@@ -612,6 +630,13 @@ int sig;
 	mustexit = 1;
 }
 
+static RETSIGTYPE sig_quit(sig)
+int sig;
+{
+	slow_shutdown = 1;
+	freeze = 1;
+}
+
 static RETSIGTYPE sig_alarm(sig)
 int sig;
 {
@@ -892,7 +917,7 @@ static int syncweb(dq, cehead)
 	  dq->sorted = 0;
 	}
 
-	if (!dq->sorted) {
+	if (!dq->sorted && dq->wrkcount > 1) {
 	  /* Sort the dq->stats[] per file ctime -- LATEST on slot 0.
 	     (we drain this queue from the END) */
 	  mytime(&now);
@@ -1543,7 +1568,8 @@ static struct ctlfile *vtxprep(cfp, file)
 	cfp->fd = -1;
 	/* sort it to get channels and channel/hosts together */
 	bcp = cfp->contents;
-	qsort((char *)offarr, opcnt, sizeof (struct offsort), bcfcn);
+	if (opcnt > 1)
+	  qsort((char *)offarr, opcnt, sizeof (struct offsort), bcfcn);
 	/*
 	 * Loop through them; whenever either channel or host changes,
 	 * make a new vertex. All such vertices must be linked together.
