@@ -108,8 +108,8 @@ static int char_array[] = {
 /*  55: `7'    */ CHR_ENT(0,1,0,1,1,1,1,0,0),
 /*  56: `8'    */ CHR_ENT(0,1,0,1,1,1,1,0,0),
 /*  57: `9'    */ CHR_ENT(0,1,0,1,1,1,1,0,0),
-/*  58: `:'    */ CHR_ENT(0,0,0,0,1,1,1,0,0),
-/*  59: `;'    */ CHR_ENT(0,0,0,0,1,1,1,0,0),
+/*  58: `:'    */ CHR_ENT(0,0,0,0,0,1,1,0,0),
+/*  59: `;'    */ CHR_ENT(0,0,0,0,0,1,1,0,0),
 /*  60: `<'    */ CHR_ENT(0,0,1,0,0,1,1,0,0),
 /*  61: `='    */ CHR_ENT(0,0,0,0,1,1,1,0,0),
 /*  62: `>'    */ CHR_ENT(0,0,1,0,0,1,1,0,0),
@@ -575,7 +575,7 @@ int strict;
 	while ((q = rfc821_char(p)) && (q > p)) {
 	  p = q;
 	}
-	if ((unsigned char)*q > 127) {
+	if (*(unsigned char*)q > 127) {
 	  rfc821_error_ptr = q;
 	  rfc821_error     = "Improper 8-bit character in string";
 	  return s;
