@@ -73,6 +73,9 @@ main(argc, argv)
 	SIGNAL_HANDLESAVE(SIGTERM, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
 	  SIGNAL_HANDLE(SIGTERM, wantout);
+	SIGNAL_HANDLESAVE(SIGQUIT, SIG_IGN, oldsig);
+	if (oldsig != SIG_IGN)
+	  SIGNAL_HANDLE(SIGQUIT, wantout);
 	SIGNAL_IGNORE(SIGPIPE);
 
 	if ((progname = strrchr(argv[0], '/')) == NULL)
@@ -102,7 +105,7 @@ main(argc, argv)
 	}
 
 	while (!getout) {
-	  char *s;
+	  char *s, *host;
 
 	  printf("#hungry\n");
 	  fflush(stdout);
@@ -121,11 +124,11 @@ main(argc, argv)
 	  if (emptyline(msgfilename, sizeof msgfilename))
 	    break;
 
-	  s = strchr(msgfilename,'\t');
-	  if (s != NULL)
-	    *s = 0;	/* Ignore the host-selector */
+	  host = strchr(msgfilename,'\t');
+	  if (host != NULL)
+	    *host++ = 0;
 
-	  dp = ctlopen(msgfilename, channel, (char *)NULL, &getout, ctlsticky, NULL);
+	  dp = ctlopen(msgfilename, channel, host, &getout, ctlsticky, NULL);
 	  if (dp != NULL) {
 	    process(dp);
 	    ctlclose(dp);
@@ -307,8 +310,10 @@ process(dp)
 	  } /* ... while() ends.. */
 	  fclose(efp);
 	} else {
+	  fputs("to <postmaster>\n",mfp); /* Default-form Cc: ... */
+	  fputs("env-end\n",mfp);
 	  for (cpp = &dfltform[0]; *cpp != NULL; ++cpp)
-	    if (*cpp == 0) {
+	    if (**cpp == '\0') {
 	      fprintf(mfp, "\tboundary=\"%s\"\n\n\n",boundarystr);
 	      fprintf(mfp, "--%s\n", boundarystr);
 	      fprintf(mfp, "Content-Type: text/plain\n");
