@@ -365,6 +365,9 @@ main(argc, argv)
 	SIGNAL_HANDLESAVE(SIGTERM, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
 	  SIGNAL_HANDLE(SIGTERM, wantout);
+	SIGNAL_HANDLESAVE(SIGQUIT, SIG_IGN, oldsig);
+	if (oldsig != SIG_IGN)
+	  SIGNAL_HANDLE(SIGQUIT, wantout);
 	SIGNAL_HANDLESAVE(SIGHUP, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
 	  SIGNAL_HANDLE(SIGHUP, wantout);
@@ -1307,7 +1310,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	char buf[2];
 	struct stat st;
 	char *fromuser;
-	int lastch;
+	int lastch = 0xFFF;
 
 	if ((fp = fdopen(fdmail, fdopmode)) == NULL) {
 	  notaryreport(NULL,NULL,NULL,NULL);
@@ -1361,7 +1364,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	{
 	  char **hdrs = has_header(rp,"Return-Path:");
 	  if (hdrs) delete_header(rp,hdrs);
-	  append_header(rp,"Return-Path: <%s>\n", fromuser);
+	  append_header(rp,"Return-Path: <%s>", fromuser);
 
 	  hdrs = has_header(rp,"To:");
 	  if (!hdrs) {
@@ -1369,7 +1372,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	    /* Sendmailism... */
 	    hdrs = has_header(rp,"Apparently-To:");
 	    if (hdrs) delete_header(rp,hdrs);
-	    append_header(rp,"Apparently-To: <%s>\n", rp->addr->link->user);
+	    append_header(rp,"Apparently-To: <%s>", rp->addr->link->user);
 	  }
 	}
 	
@@ -1399,7 +1402,7 @@ putmail(dp, rp, fdmail, fdopmode, timestring, file)
 	}
 	lastch = appendlet(dp, rp, fp, file, is_mime);
 
-	if (lastch < 0 || ferror(fp) || fflush(fp) != 0) {
+	if (lastch < -128 || ferror(fp) || fflush(fp) != 0) {
 	  int err;
       write_failure:
 	  err = errno;
@@ -1977,7 +1980,7 @@ appendlet(dp, rp, fp, file, ismime)
 	      if (writemimeline(fp, s0, linelen, &lastch) != linelen) {
 		DIAGNOSTIC(rp, EX_IOERR,
 			   "write to \"%s\" failed", file);
-		return -1;
+		return -256;
 	      }
 	      s0 = s;
 	    }
@@ -2005,14 +2008,14 @@ appendlet(dp, rp, fp, file, ismime)
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
  	      MFPCLOSE;
-	      return -1;
+	      return -256;
 	    }
 	  }
 	  if (i == EOF && !feof(mfp)) {
 	    DIAGNOSTIC(rp, EX_IOERR,
 		       "read error from message file", 0);
 	    MFPCLOSE;
-	    return -1;
+	    return -256;
 	  }
 
 	  MFPCLOSE;
@@ -2024,7 +2027,7 @@ appendlet(dp, rp, fp, file, ismime)
 	    if (writebuf(fp, buffer, readalready, &lastch) != readalready) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
-	      return -1;
+	      return -256;
 	    }
 	    rp->status = EX_OK;
 	    return lastch;
@@ -2037,12 +2040,12 @@ appendlet(dp, rp, fp, file, ismime)
 	    if (i < 0) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "read error from message file", 0);
-	      return -1;
+	      return -256;
 	    }
 	    if (writebuf(fp, buffer, i, &lastch) != i) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
-	      return -1;
+	      return -256;
 	    }
 	    readalready = i;
 	    bufferfull++;
@@ -2065,7 +2068,7 @@ appendlet(dp, rp, fp, file, ismime)
 	    if (writemimeline(fp, s, i, &lastch) != i) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
-	      return -1;
+	      return -256;
 	    }
 	    s = s2;
 	  }
@@ -2074,7 +2077,7 @@ appendlet(dp, rp, fp, file, ismime)
 	  if (writebuf(fp, s, dp->let_end - s, &lastch) != (dp->let_end - s)) {
 	      DIAGNOSTIC(rp, EX_IOERR,
 			 "write to \"%s\" failed", file);
-	      return -1;
+	      return -256;
 	  }
 	}
 #endif
@@ -2528,7 +2531,6 @@ return_receipt (dp, retrecptaddr, uidstr)
 	FILE *mfp, *efp;
 	struct rcpt *rp;
 	int n;
-	char lastchar;
 	char *boundarystr = buildboundary("return-receipt");
 	struct passwd *pw;
 	int uid;
@@ -2601,10 +2603,10 @@ return_receipt (dp, retrecptaddr, uidstr)
 	  fclose(efp);
 	} else {
 	  for (cpp = &dfltform[0]; *cpp != NULL; ++cpp)
-	    if (*cpp == 0) {
+	    if (*cpp[0] == 0) {
 	      fprintf(mfp, "\tboundary=\"%s\"\n\n\n",boundarystr);
 	      fprintf(mfp, "--%s\n", boundarystr);
-	      fprintf(mfp, "Content-Type: text/plain\n");
+	      fprintf(mfp, "Content-Type: text/plain\n\n");
 	    } else
 	      fprintf(mfp, "%s\n", *cpp);
 	}
