@@ -97,21 +97,23 @@ struct maildesc {
 	char	*argv[20];
 };
 
-#define	MO_FFROMFLAG		    01
-#define	MO_RFROMFLAG		    02
-#define	MO_NORESETUID		    04
-#define	MO_STRIPQUOTES		   010
-#define	MO_MANYUSERS		   020
-#define	MO_RETURNPATH		   040
-#define	MO_UNIXFROM		  0100
-#define	MO_HIDDENDOT		  0200 /* SMTP dot-duplication */
-#define	MO_ESCAPEFROM		  0400
-#define	MO_STRIPHIBIT		 01000
-#define	MO_REMOTEFROM		 02000
-#define	MO_CRLF			 04000
-#define MO_BSMTP		010000 /* BSMTP-wrapping -- with HIDDENDOT.. */
-#define MO_BESMTP		020000 /* Extended BSMTP -- SIZE+8BITMIME    */
-#define MO_BEDSMTP		040000 /* EBSMTP + DSN */
+#define	MO_FFROMFLAG		0x00001
+#define	MO_RFROMFLAG		0x00002
+#define	MO_NORESETUID		0x00004
+#define	MO_STRIPQUOTES		0x00008
+#define	MO_MANYUSERS		0x00010
+#define	MO_RETURNPATH		0x00020
+#define	MO_UNIXFROM		0x00040
+#define	MO_HIDDENDOT		0x00080 /* SMTP dot-duplication */
+#define	MO_ESCAPEFROM		0x00100
+#define	MO_STRIPHIBIT		0x00200
+#define	MO_REMOTEFROM		0x00400
+#define	MO_CRLF			0x00800
+#define MO_BSMTP		0x01000 /* BSMTP-wrapping -- with HIDDENDOT.. */
+#define MO_BESMTP		0x02000 /* Extended BSMTP -- SIZE+8BITMIME    */
+#define MO_BEDSMTP		0x04000 /* EBSMTP + DSN */
+#define MO_WANTSDATE		0x08000 /* Wants "Date:" -header */
+#define MO_WANTSFROM		0x10000 /* Wants "From:" -header */
 
 struct exmapinfo {
 	int	origstatus;
@@ -166,6 +168,9 @@ main(argc, argv)
 	SIGNAL_HANDLESAVE(SIGTERM, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
 	  SIGNAL_HANDLE(SIGTERM, wantout);
+	SIGNAL_HANDLESAVE(SIGQUIT, SIG_IGN, oldsig);
+	if (oldsig != SIG_IGN)
+	  SIGNAL_HANDLE(SIGQUIT, wantout);
 	SIGNAL_HANDLESAVE(SIGHUP, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
 	  SIGNAL_HANDLE(SIGHUP, wantout);
@@ -618,7 +623,7 @@ deliver(dp, mp, startrp, endrp, verboselog)
 	  if (hdrs) delete_header(startrp, hdrs);
 	  if (strcmp(startrp->addr->link->channel,"error")==0)
 	    uu = "";
-	  append_header(startrp,"Return-Path: <%s>\n", uu);
+	  append_header(startrp,"Return-Path: <%s>", uu);
 	}
 	if (mp->flags & MO_CRLF)
 	  writeheaders(startrp, tafp, "\r\n", convertmode, maxwidth);
@@ -1129,6 +1134,10 @@ readsmcf(file, mailer)
 	/* process mailer option flags */
 	for (;isascii(*cp) && !isspace(*cp); ++cp) {
 	  switch (*cp) {
+	  case 'D':		/* this mailer wants a Date: line */
+	    	m.flags |= MO_WANTSDATE;	break;
+	  case 'F':		/* this mailer wants a From: line */
+	    	m.flags |= MO_WANTSFROM;	break;
 	  case '7':	m.flags |= MO_STRIPHIBIT;	break;
 	  case 'E':	m.flags |= MO_ESCAPEFROM;	break;
 	  case 'P':	m.flags |= MO_RETURNPATH;	break;
@@ -1149,8 +1158,6 @@ readsmcf(file, mailer)
 			break;
 	  case 'A':		/* arpanet-compatibility */
 	  case 'C':		/* canonicalize remote hostnames */
-	  case 'D':		/* this mailer wants a Date: line */
-	  case 'F':		/* this mailer wants a From: line */
 	  case 'I':		/* talking to a clone of I */
 	  case 'L':		/* limit line length */
 	  case 'M':		/* this mailer wants a Message-Id: line */
