@@ -166,7 +166,6 @@ int readalready = 0;		/* does buffer contain valid message data? */
 #endif
 int wantreserved = 0;		/* open connection on secure (reserved) port */
 int statusreport = 0;		/* put status reports on the command line */
-int noMX = 0;			/* do not use MX records to deliver mail */
 int firstmx = 0;		/* [Matti Aarnio] error in smtpwrite("HELO"..*/
 int mxcount = 0;
 int force_8bit = 0;		/* Claim to the remote to be 8-bit system, even
@@ -176,6 +175,9 @@ int keep_header8 = 0;		/* Don't do "MIME-2" to the headers */
 int checkwks = 0;
 FILE *logfp = NULL;
 extern int nobody;
+char *localidentity = NULL;	/* If we are wanted to bind some altenate
+				   interface than what the default is thru
+				   normal kernel mechanisms.. */
 int daemon_uid = -1;
 int first_uid = 0;		/* Make the opening connect with the UID of the
 				   sender (atoi(rp->addr->misc)), unless it is
@@ -258,14 +260,14 @@ extern int deliver __((SmtpState *SS, struct ctldesc *dp, struct rcpt *startrp,
 extern int writebuf __((SmtpState *SS, char *buf, int len));
 extern int writemimeline __((SmtpState *SS, char *buf, int len, int convertmode, int* lastchp));
 extern int appendlet __((SmtpState *SS, struct ctldesc *dp, int hsize, int msize, int convertmode));
-extern int smtpopen __((SmtpState *SS, char *host));
-extern int smtpconn __((SmtpState *SS,  char *host));
+extern int smtpopen __((SmtpState *SS, char *host, int noMX));
+extern int smtpconn __((SmtpState *SS,  char *host, int noMX));
 extern int smtp_ehlo __((SmtpState *SS, char *strbuf));
 extern int ehlo_check __((SmtpState *SS, char *buf));
 extern void smtp_flush __((SmtpState *SS));
 extern int smtp_sync __((SmtpState *SS, int));
 extern int smtpwrite __((SmtpState *SS, int saverpt, char *linebuf, struct rcpt *syncrp));
-extern int process __((SmtpState *SS, struct ctldesc*, int, char*));
+extern int process __((SmtpState *SS, struct ctldesc*, int, char*, int));
 
 extern int check_7bit_cleanness __((struct ctldesc *dp));
 extern void notarystatsave __((SmtpState *SS, char *smtpstatline, char *status));
@@ -340,12 +342,14 @@ main(argc, argv)
 	int i, fd, errflg, c, smtpstatus;
 	int need_host = 0;
 	int idle;
+	int noMX = 0;
 	SmtpState SS;
 	struct ctldesc *dp;
 #ifdef	BIND
 	int checkmx = 0;	/* check all destination hosts for MXness */
 #endif	/* BIND */
 	RETSIGTYPE (*oldsig)();
+	char *smtphost, *punthost = NULL;
 
 	pid = getpid();
 	ipaddress[0] = '\0';
@@ -354,16 +358,11 @@ main(argc, argv)
 	cmdline = &argv[0][0];
 	eocmdline = cmdline;
 
-	SS.ehlo_capabilities = 0;
-	SS.within_ehlo	     = 0;
+	memset(&SS,0,sizeof(SS));
 	SS.main_esmtp_on_banner = -1;
 	SS.servport      = -1;
 	SS.smtp_bufsize  = 8*1024;
-	SS.smtp_outcount = 0;
 	SS.ehlo_sizeval  = -1;
-	SS.smtpfp	 = NULL;
-	SS.pipebuf       = NULL;
-	SS.pipecmds	 = NULL;
 
 	for (i = 0; i < argc; ++i)
 	  eocmdline += strlen(argv[i]) + 1;
@@ -377,6 +376,9 @@ main(argc, argv)
 	SIGNAL_HANDLESAVE(SIGTERM, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
 	  SIGNAL_HANDLE(SIGTERM, wantout);
+	SIGNAL_HANDLESAVE(SIGQUIT, SIG_IGN, oldsig);
+	if (oldsig != SIG_IGN)
+	  SIGNAL_HANDLE(SIGQUIT, wantout);
 	SIGNAL_HANDLESAVE(SIGHUP, SIG_IGN, oldsig);
 	if (oldsig != SIG_IGN)
 	  SIGNAL_HANDLE(SIGHUP, wantout);
@@ -397,7 +399,7 @@ main(argc, argv)
 	myhostname[0] = '\0';
 	remotemsg[0] = '\0';
 	remotehost[0] = '\0';
-	while ((c = getopt(argc, argv, "c:deh:l:p:rsvxDEHPT:VW78")) != EOF) {
+	while ((c = getopt(argc, argv, "c:deh:l:p:rsvxDEF:L:HPT:VW78")) != EOF) {
 	  switch (c) {
 	  case 'c':		/* specify channel scanned for */
 	    channel = strdup(optarg);
@@ -443,6 +445,13 @@ main(argc, argv)
 				   has "ESMTP" on its banner */
 	    SS.main_esmtp_on_banner = 0;
 	    break;
+	  case 'F':		/* Send all SMTP sessions to that host,
+				   possibly set also '-x' to avoid MXes! */
+	    punthost = optarg;
+	    break;
+	  case 'L':		/* Specify which local identity to use */
+	    localidentity = optarg;
+	    break;
 	  case 'T':		/* specify Timeout in seconds */
 	    if ((timeout = atoi(optarg)) < 5) {
 	      fprintf(stderr,
@@ -498,7 +507,7 @@ main(argc, argv)
 
 	if (conndebug && !debug && host) {
 	  firstmx = 0;
-	  smtpconn(&SS, host);
+	  smtpconn(&SS, host, noMX);
 	  exit(0);
 	}
 
@@ -513,7 +522,7 @@ main(argc, argv)
 	  logfp = NULL;
 
 	if (logfp)
-	  setvbuf(logfp, (char *)NULL, _IOLBF, 0);
+	  setvbuf(logfp, NULL, _IOLBF, 0);
 
 	getnobody();
 	getdaemon();
@@ -613,16 +622,23 @@ main(argc, argv)
 	    continue;
 	  }
 	
+	  if (punthost)
+	    smtphost = punthost;
+	  else
+	    smtphost = host;
+
 	  if (dp->verbose) {
+	    if (SS.verboselog)
+	      fclose(SS.verboselog);
 	    SS.verboselog = (FILE *)fopen(dp->verbose,"a");
 	    if (SS.verboselog)
-	      setbuf(SS.verboselog,NULL);
+	      setvbuf(SS.verboselog, NULL, _IONBF, 0);
 	  }
 	  if (!SS.smtpfp) {
 	    if ((first_uid = atoi(dp->senders->misc)) < 0 ||
 		first_uid == nobody)
 	      first_uid = daemon_uid;
-	    smtpstatus = smtpopen(&SS, host);
+	    smtpstatus = smtpopen(&SS, smtphost, noMX);
 	    if (smtpstatus != EX_OK) {
 	      /* SMTPOPEN() made diagnostics data -- we report.. */
 	      struct rcpt *rp;
@@ -636,7 +652,7 @@ main(argc, argv)
 	    smtpstatus = EX_OK; /* Already open, we have a new job file */
 	
 	  if (smtpstatus == EX_OK)
-	    smtpstatus = process(&SS, dp, smtpstatus, host);
+	    smtpstatus = process(&SS, dp, smtpstatus, smtphost, noMX);
 	  if (SS.verboselog)
 	    fclose(SS.verboselog);
 	  SS.verboselog = NULL;
@@ -666,11 +682,12 @@ main(argc, argv)
 }
 
 int
-process(SS, dp, smtpstatus, host)
+process(SS, dp, smtpstatus, host, noMX)
 	SmtpState *SS;
 	struct ctldesc *dp;
 	int smtpstatus;
 	char *host;
+	int noMX;
 {
 	struct rcpt *rp, *rphead;
 	int loggedid;
@@ -697,7 +714,7 @@ process(SS, dp, smtpstatus, host)
 		if ((first_uid = atoi(dp->senders->misc)) < 0 ||
 		    first_uid == nobody)
 		  first_uid = daemon_uid;
-		openstatus = smtpopen(SS, host);
+		openstatus = smtpopen(SS, host, noMX);
 	      }
 	      if (openstatus == EX_OK)
 		smtpstatus = deliver(SS, dp, rphead, rp->next);
@@ -824,6 +841,8 @@ deliver(SS, dp, startrp, endrp)
 	  more_rp = NULL;
 	}
 
+	smtp_flush(SS); /* Flush in every case */
+
 	if (strcmp(startrp->addr->link->channel,"error")==0)
 	  sprintf(SMTPbuf, "MAIL From:<>");
 	else
@@ -846,11 +865,9 @@ deliver(SS, dp, startrp, endrp)
 	    s += strlen(s);
 	  }
 	  if (startrp->desc->dsnretmode != NULL)
-	    sprintf(s, "RET=%s", startrp->desc->dsnretmode);
+	    sprintf(s, " RET=%s", startrp->desc->dsnretmode);
 	}
 
-	smtp_flush(SS); /* Flush in every case */
-
 	time(&env_start); /* Mark the timestamp */
 
 	/* MAIL FROM:<...> -- pipelineable.. */
@@ -1180,7 +1197,7 @@ appendlet(SS, dp,  hsize, msize, convertmode)
 		return EX_IOERR;
 	      }
 	      sprintf(remotemsg,
-		      "smtp; 500 (let_buffer write IO-error! [%s] DATA %d/%d [%d%%])",
+		      "smtp; 500 (let_buffer write IO-error[1]! [%s] DATA %d/%d [%d%%])",
 		      strerror(errno), hsize, msize, (hsize*100+msize/2)/msize);
 	      return EX_IOERR;
 	    }
@@ -1240,7 +1257,7 @@ appendlet(SS, dp,  hsize, msize, convertmode)
 		     hsize, msize, (hsize*100+msize/2)/msize);
 	    if (rc != i) {
 	      sprintf(remotemsg,
-		      "500 (let_buffer write IO-error! [%s] DATA %d/%d [%d%%])",
+		      "500 (let_buffer write IO-error[2]! [%s] DATA %d/%d [%d%%])",
 		      strerror(errno), hsize, msize, (hsize*100+msize/2)/msize);
 	      MFPCLOSE
 	      return EX_IOERR;
@@ -1268,7 +1285,7 @@ appendlet(SS, dp,  hsize, msize, convertmode)
 	      return EX_IOERR;
 	    }
 	    sprintf(remotemsg,
-		    "500 (let_buffer write IO-error! [%s] DATA %d/%d [%d%%])",
+		    "500 (let_buffer write IO-error[3]! [%s] DATA %d/%d [%d%%])",
 		    strerror(errno), hsize, msize, (hsize*100+msize/2)/msize);
 #ifndef	HAVE_MMAP
 	    if (bufferfull > 1) readalready = 0;
@@ -1569,9 +1586,10 @@ char *str;
 
 
 int
-smtpopen(SS, host)
+smtpopen(SS, host, noMX)
 	char *host;
 	SmtpState *SS;
+	int noMX;
 {
 	int i;
 	char SMTPbuf[1000];
@@ -1588,7 +1606,7 @@ smtpopen(SS, host)
 	  SS->ehlo_sizeval = 0;
 	  SS->rcpt_limit = 100; /* Max number of recipients per message */
 
-	  if ((i = smtpconn(SS, host)) != EX_OK)
+	  if ((i = smtpconn(SS, host, noMX)) != EX_OK)
 	    return i;
 	  if (SS->esmtp_on_banner) {
 	    /* Either it is not tested, or it is explicitely
@@ -1598,7 +1616,7 @@ smtpopen(SS, host)
 	    if (i == EX_TEMPFAIL) {
 	      /* There are systems, which hang up on us, when we
 		 greet them with an "EHLO".. Do here a normal "HELO".. */
-	      if ((i = smtpconn(SS, host)) != EX_OK)
+	      if ((i = smtpconn(SS, host, noMX)) != EX_OK)
 		return i;
 	      i = EX_TEMPFAIL;
 	    }
@@ -1619,7 +1637,7 @@ smtpopen(SS, host)
 	    }
 	    if (i == EX_TEMPFAIL) {
 	      /* Ok, sometimes EHLO+HELO cause crash, open and do HELO only */
-	      if ((i = smtpconn(SS, host)) != EX_OK)
+	      if ((i = smtpconn(SS, host, noMX)) != EX_OK)
 		return i;
 	      i = smtpwrite(SS, 1, SMTPbuf, NULL);
 	      if (i == EX_TEMPFAIL && SS->smtpfp) {
@@ -1637,9 +1655,10 @@ smtpopen(SS, host)
 }
 
 int
-smtpconn(SS, host)
+smtpconn(SS, host, noMX)
 	SmtpState *SS;
 	char *host;
+	int noMX;
 {
 	int	i, r, retval, smtpfd;
 	struct in_addr	ipaddr;
@@ -1973,9 +1992,6 @@ vcsetup(SS,ipaddrcp, ipaddrlen, fdp, hostname)
 	  strcpy(remotehost,hostname);
 	  report(SS,"connecting to [%s]",ipaddress);
 	}
-	/* try grabbing a port */
-	sad.sin_family = AF_INET;
-	sad.sin_addr.s_addr = 0;
 	if ((s = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
 	  se = strerror(errno);
 	  sprintf(remotemsg, "smtp; 500 (Internal error, socket: %s)", se);
@@ -1988,8 +2004,25 @@ vcsetup(SS,ipaddrcp, ipaddrlen, fdp, hostname)
 	    fprintf(logfp,"%s#\t(Internal error, socket: %s)\n",logtag(),se);
 	  return EX_TEMPFAIL;
 	}
+
+	sad.sin_addr.s_addr = 0;
+	if (localidentity != NULL) {
+	  /* Uh... Somebody wants us to do special hoops...
+	     ... to bind some of our alternate IP addresses,
+	     for example.. */
+	  sad.sin_addr.s_addr = inet_addr(localidentity);
+	  if ((int)(sad.sin_addr.s_addr) == -1) {
+	    struct hostent *hp = gethostbyname(localidentity);
+	    sad.sin_addr.s_addr = 0;
+	    if (hp) /* We try ONLY the first address. */
+	      memcpy((void*)&sad.sin_addr, hp->h_addr_list[0], 4);
+	  }
+	}
+
 	if (wantreserved && getuid() == 0) {
+	  /* try grabbing a port */
 	  for (p = IPPORT_RESERVED-1; p >= (u_short)(IPPORT_RESERVED/2); --p) {
+	    sad.sin_family = AF_INET;
 	    sad.sin_port = htons(p);
 	    if (bind(s, (struct sockaddr *)&sad, sizeof sad) >= 0)
 	      break;
@@ -2017,7 +2050,14 @@ vcsetup(SS,ipaddrcp, ipaddrlen, fdp, hostname)
 	      fprintf(logfp,"%s#\t(Internal error, too many busy ports)\n", logtag());
 	    return EX_TEMPFAIL;
 	  }
+	} else if (localidentity != NULL) {
+	  /* Ok, it wasn't a desire for any PRIVILEGED port, just
+	     binding on the specific IP will be accepted. */
+	  bind(s, (struct sockaddr *)&sad, sizeof sad);
+	  /* If it fails, what could we do ? */
 	}
+
+	sad.sin_family = AF_INET;
 	memcpy((char *)&sad.sin_addr, ipaddrcp, ipaddrlen);
 	if (SS->servport < 0)
 	  sad.sin_port = htons((u_short)IPPORT_SMTP);
@@ -2388,7 +2428,7 @@ int smtp_sync(SS,r)
 	  eof = SS->pipebuf + SS->pipebufsize;
 	  for (eol = s; eol < eof; ++eol)
 	    if (*eol == '\n') break;
-	  if (*eol == '\n') {
+	  if (eol < eof && *eol == '\n') {
 	    ++eol; /* points to char AFTER the newline */
 	    if (debug && logfp)
 	      fprintf(logfp,"%s#\t(pipebufsize=%d, s=%d, eol=%d)\n",
@@ -2407,6 +2447,7 @@ int smtp_sync(SS,r)
 		fprintf(SS->verboselog,"Timeout (%d sec) while waiting responces from remote\n",timeout);
 	      break;
 	    }
+	  reread_line:
 	    len = read(infd,buf,sizeof(buf));
 	    err = errno;
 	    
@@ -2414,7 +2455,7 @@ int smtp_sync(SS,r)
 	      /* Some error ?? How come ?
 		 We have select() confirmed input! */
 	      if (err == EINTR)
-		goto rescan_line;
+		goto reread_line;
 	      /* XX: what to do with the error ? */
 	      if (logfp)
 		fprintf(logfp,"%s#\tRemote gave error %d (%s) while %d responces missing\n",
@@ -2457,10 +2498,10 @@ int smtp_sync(SS,r)
 	      SS->pipebufsize += len;
 	      goto rescan_line;
 	    }
-	  } /* ... globbing more input */
+	  } /* -- endif -- ... globbing more input */
 
 	  p = eol-1;		  /* The '\n' at the end of the line	*/
-	  if (p[-1] == '\r') --p; /* "\r\n" ?				*/
+	  if (p > s && p[-1] == '\r') --p; /* "\r\n" ?			*/
 	  *p = 0;
 
 	  if (logfp != NULL) {
@@ -2546,8 +2587,7 @@ SmtpState *SS;
 	  for (;;) {
 	    r = read(infd, buf, sizeof buf);
 	    if (r == 0) break; /* EOF ?! */
-	    if (r < 0)
-	      break;
+	    if (r < 0)  break;
 	    if (SS->pipebuf == NULL) {
 	      SS->pipebufspace = 240;
 	      while (SS->pipebufspace < r+1)
@@ -2562,8 +2602,8 @@ SmtpState *SS;
 	    memcpy(SS->pipebuf+SS->pipebufsize,buf,r);
 	    SS->pipebufsize += r;
 	    SS->block_written = 0; /* We drain the accumulated input here,
-				  and can thus mark this draining
-				  unneeded for a while. */
+				      and can thus mark this draining
+				      unneeded for a while. */
 	  }
 	  fd_blockingmode(infd);
 	  /* Continue the processing... */
@@ -2632,6 +2672,7 @@ smtpwrite(SS, saverpt, strbuf, syncrp)
 	      gotalarm = 0;
 	    }
 
+	    /* Read possible reponses into response buffer.. */
 	    pipeblockread(SS);
 
 	    memcpy(buf,strbuf,len-2);
@@ -2987,7 +3028,7 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 	n = res_send((void*)&qbuf, qlen, (void*)&answer, sizeof answer);
 	if (n < 0) {
 	  sprintf(remotemsg,
-		  "smtp; 466 (Internal: res_send failed on host: %s)",host);
+		  "smtp; 466 (No DNS response for host: %s)",host);
 	  notaryreport(NULL,FAILED,"5.4.3 (DNS-failure)",remotemsg);
 	  return EX_TEMPFAIL;
 	}
@@ -3056,13 +3097,15 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 	    break;
 	  cp += n;
 	  type = _getshort(cp);
-	  cp += sizeof(u_short);
+	  cp += 2;
 	  /*
 	     class = _getshort(cp);
 	     */
-	  cp += sizeof(u_short) + sizeof(u_long);
+	  cp += (2 + 4); /* "short" + "long" -- but keep in mind that
+			    some machines have "funny" ideas about
+			    "long" -- those 64-bit ones... */
 	  n = _getshort(cp);
-	  cp += sizeof(u_short);
+	  cp += 2;
 	  if (type == T_CNAME) {
 	    cp += dn_expand((msgdata *)&answer, eom, cp,
 			    (void*)realname, sizeof realname);
@@ -3073,7 +3116,7 @@ getmxrr(SS, host, mx, maxmx, depth, mxcntp)
 	    continue;
 	  }
 	  mx[nmx].pref = _getshort(cp);
-	  cp += sizeof(u_short); /* MX preference value */
+	  cp += 2; /* MX preference value */
 	  n = dn_expand((msgdata *)&answer, eom, cp, (void*)buf, sizeof buf);
 	  if (n < 0)
 	    break;
@@ -3238,7 +3281,7 @@ rightmx(spec_host, addr_host)
 	  return 0;
 	}
 	for (i = 0; mxh[i].host != NULL; ++i) {
-	  if (cistrcmp(mxh[i].host, remotehost) == 0) {
+	  if (cistrcmp((char*)mxh[i].host, remotehost) == 0) {
 	    while (mxh[i].host != NULL)
 	      free(mxh[i++].host);
 	    return 1;
